@@ -442,6 +442,11 @@ class _StructureLayer:
 
 	def objectHasField(self, id, field):
 		"""Returns True if object has given field (with any type of value)"""
+
+		# if field points to root, object definitely has it
+		if len(field.name) == 0:
+			return True
+
 		types = self.getValueTypes(id, field)
 		field_copy = Field(self._engine, field.name)
 		for type in types:
@@ -504,6 +509,8 @@ class LogicLayer:
 					name_copy.append(next)
 					while len(name_copy) > 0:
 						tmp_field.py_value = (dict() if isinstance(name_copy[-1], str) else list())
+						if len(tmp_field.name) > 0 and isinstance(tmp_field.name[-1], int):
+							self._fillWithNones(id, tmp_field)
 						self._setFieldValue(id, tmp_field)
 						tmp_field.name.append(name_copy.pop())
 
@@ -546,6 +553,20 @@ class LogicLayer:
 			# shift numbers of all elements in list
 			self._structure.renumberList(id, target_field, fld, shift)
 
+	def _fillWithNones(self, id, path):
+		max = self._structure.getMaxListIndex(id, path)
+		if max is None:
+			start = 0
+		else:
+			start = max + 1
+		end = path.name[-1]
+
+		path_copy = Field(self._engine, path.name)
+		for i in range(start, end):
+			path_copy.name[-1] = i
+			path_copy.py_value = None
+			self._setFieldValue(id, path_copy)
+
 	def _modifyFields(self, id, path, fields, remove_conflicts):
 		"""Store values of given fields"""
 
@@ -554,9 +575,18 @@ class LogicLayer:
 			for field in self._structure.getFieldsList(id, path, exclude_self=False):
 				self._structure.deleteValues(id, field, path.columns_condition)
 		else:
-		# path does not exist and is not root - check for list/map conflicts
+		# path does not exist and is not root
+
+			# check for list/map conflicts
 			self._checkForConflicts(id, path, remove_conflicts)
 
+			# fill autocreated list elements (if any) with Nones
+			field_copy = Field(self._engine, path.name)
+			while len(field_copy.name) > 0:
+				if isinstance(field_copy.name[-1], int):
+					self._fillWithNones(id, field_copy)
+				field_copy.name.pop()
+
 		# store field values
 		for field in fields:
 			field.name = path.name + field.name
@@ -589,8 +619,9 @@ class LogicLayer:
 			raise interface.LogicError("Object " + str(request.id) + " does not exist")
 
 		if request.path is not None:
-			types = self._structure.getValueTypes(request.id, request.path)
-			if len(types) == 0:
+			#types = self._structure.getValueTypes(request.id, request.path)
+			#if len(types) == 0:
+			if not self._structure.objectHasField(request.id, request.path):
 				raise interface.LogicError("Object " + str(request.id) +
 					" does not have field " + str(request.path.name))
 
@@ -645,15 +676,21 @@ class LogicLayer:
 		# where request.path is pointing to
 		parent_field = Field(self._engine, request.path.name[:-1])
 		parent = self._structure.getFieldValue(request.id, parent_field)
-		types = self._structure.getValueTypes(request.id, parent_field)
 
-		if len(parent) == 0:
-			# try to autovivify list
-			new_val = Field(self._engine, [], list())
-			self._modifyFields(request.id, parent_field,
-				[new_val], remove_conflicts=request.remove_conflicts)
-		elif parent[0].py_value != list():
-			raise interface.StructureError("Cannot insert to non-list")
+		if len(parent) == 0 or parent[0].py_value != list():
+			if len(parent) == 0 or request.remove_conflicts:
+			# try to autocreate list
+				new_val = Field(self._engine, [], list())
+				self._modifyFields(request.id, parent_field,
+					[new_val], remove_conflicts=request.remove_conflicts)
+			else:
+			# in this case we can raise more meaningful error
+				raise interface.StructureError("Cannot insert to non-list")
+
+		# if path does not point to beginning of the list, fill
+		# missing elements with Nones
+		if request.path.name[-1] is not None and request.path.name[-1] > 0:
+			self._fillWithNones(request.id, request.path)
 
 		target_col = len(request.path.name) - 1 # last column in name of target field
 
