@@ -5,8 +5,10 @@ import string
 import copy
 import traceback
 import time
+import sys
 
 import brain
+from brain.data import *
 
 STARTING_DEPTH = 5 # starting data depth of objects
 MAX_DEPTH = 5 # maximum depth of data structures created during test
@@ -14,7 +16,8 @@ MAX_ELEMENTS_NUM = 3 # maximum number of elements in created lists/dictionaries
 STOP_DATA_GENERATION = 0.6 # probability of stopping data generation on each level of structure
 STOP_PATH_GENERATION = 0.1 # probability of stopping path generation on each level of structure
 NONE_PROBABILITY = 0.3 # probability of last None appearance in the path of insert request
-
+READ_BY_MASK_PROBABILITY = 0.6 # probability of using masks in read()
+CONFLICTING_PATH_PROBABILITY = 0.2 # probability of using conflicting path when modifying/inserting
 
 class FakeConnection:
 	"""
@@ -26,34 +29,42 @@ class FakeConnection:
 		self._id_counter = 0
 		self._root = {}
 
-	def _getPath(self, obj, path):
-		"""Get pointer to data structure with given path"""
-		if len(path) == 1:
-			return obj[path[0]]
-		else:
-			return self._getPath(obj[path[0]], path[1:])
-
 	def _deleteAll(self, obj, path):
 		"""Delete all values from given path"""
-		if len(path) == 1:
-			del obj[path[0]]
-		else:
-			self._deleteAll(obj[path[0]], path[1:])
+		if path[0] is None and isinstance(obj, list):
+			if len(path) == 1:
+				obj[:] = []
+			else:
+				for i in range(len(obj)):
+					self._deleteAll(obj[i], path[1:])
+		elif (isinstance(path[0], str) and isinstance(obj, dict) and path[0] in obj.keys()) or \
+				(isinstance(path[0], int) and isinstance(obj, list) and path[0] < len(obj)):
+			if len(path) == 1:
+				del obj[path[0]]
+			else:
+				self._deleteAll(obj[path[0]], path[1:])
 
 	def create(self, data, path=None):
 		if path is None:
 			path = []
 		self._id_counter += 1
-		_saveTo(self._root, self._id_counter, path, data)
+		saveToTree(self._root, self._id_counter, path, data)
 		return self._id_counter
 
 	def modify(self, id, path, value, remove_conflicts=False):
-		_saveTo(self._root, id, path, value)
+		saveToTree(self._root, id, path, value, remove_conflicts=remove_conflicts)
 
 	def insertMany(self, id, path, values, remove_conflicts=False):
-		target = self._getPath(self._root, [id] + path[:-1])
+		if remove_conflicts:
+			self.modify(id, path[:-1], [], remove_conflicts=True)
+
+		target = getNodeByPath(self._root[id], path[:-1])
 		index = path[-1]
 
+		if index is not None and index > len(target):
+			for i in range(len(target), index):
+				target.append(None)
+
 		if index is None:
 			for value in values:
 				target.append(value)
@@ -63,11 +74,26 @@ class FakeConnection:
 
 	def deleteMany(self, id, paths=None):
 		for path in paths:
-			self._deleteAll(self._root, [id] + path)
+			self._deleteAll(self._root[id], path)
 
-	def read(self, id):
-		return self._root[id]
+	def read(self, id, path=None, masks=None):
+		if path is None:
+			path = []
 
+		res = getNodeByPath(self._root, [id] + path)
+
+		if masks is not None:
+			fields = treeToPaths(res)
+			res_fields = []
+			for field_path, value in fields:
+				for mask in masks:
+					if pathMatchesMask(field_path, mask):
+						res_fields.append((field_path, value))
+						break
+
+			res = pathsToTree(res_fields)
+
+		return res
 
 # Auxiliary functions
 
@@ -126,6 +152,18 @@ def getRandomDefinitePath(root):
 		path = getRandomPath(root)
 	return path
 
+def getRandomConflictingPath(root):
+	path = []
+	while len(path) == 0:
+		path = getRandomDefinitePath(root)
+
+	i = random.randint(0, len(path) - 1)
+	if isinstance(path[i], str):
+		path[i] = 0
+	else:
+		path[i] = getRandomString()
+	return path
+
 def getRandomDeletePath(root):
 	"""
 	Returns random path for delete request -
@@ -134,7 +172,7 @@ def getRandomDeletePath(root):
 	"""
 	path = []
 	while len(path) == 0:
-		path = getRandomDefinitePath(root)
+		path = getRandomMask(root)
 	return path
 
 def getRandomInsertPath(root):
@@ -148,6 +186,14 @@ def getRandomInsertPath(root):
 		path = getRandomPath(root)
 	if random.random() < NONE_PROBABILITY:
 		path[-1] = None
+
+	return path
+
+def getRandomMask(root):
+	path = getRandomPath(root)
+	for i, e in enumerate(path):
+		if not isinstance(e, str) and random.random() < NONE_PROBABILITY:
+			path[i] = None
 	return path
 
 def listInData(data):
@@ -161,6 +207,14 @@ def listInData(data):
 	else:
 		return False
 
+def hasNonTrivialDefinitePath(data):
+	if isinstance(data, list) and len(data) > 0:
+		return True
+	elif isinstance(data, dict) and len(data.keys()) > 0:
+		return True
+
+	return False
+
 
 class RandomAction:
 	"""Class, representing an action on a given object"""
@@ -171,7 +225,8 @@ class RandomAction:
 		args_constructors = {
 			'modify': self._constructModifyArgs,
 			'insertMany': self._constructInsertArgs,
-			'deleteMany': self._constructDeleteArgs
+			'deleteMany': self._constructDeleteArgs,
+			'read': self._constructReadArgs
 		}
 
 		if type(self._obj_contents) not in [list, dict] or len(self._obj_contents) == 0:
@@ -185,24 +240,57 @@ class RandomAction:
 		args_constructors[self._method]()
 
 	def dump(self, verbosity):
-		return (str(self) if verbosity > 3 else self._method)
+		return (str(self) if verbosity > 3 else self.getMethod())
 
 	def _constructModifyArgs(self):
-		self._args = (getRandomDefinitePath(self._obj_contents),
-			getRandomData(MAX_DEPTH))
+		data = getRandomData(MAX_DEPTH)
+		if random.random() < CONFLICTING_PATH_PROBABILITY and \
+				hasNonTrivialDefinitePath(self._obj_contents):
+			path = getRandomConflictingPath(self._obj_contents)
+			remove_conflicts = True
+		else:
+			path = getRandomDefinitePath(self._obj_contents)
+			remove_conflicts = False
+
+		self._args = (path, data, remove_conflicts)
 
 	def _constructInsertArgs(self):
 		elems = random.randint(1, MAX_ELEMENTS_NUM)
+		path = getRandomInsertPath(self._obj_contents)
+
+		if random.random() < CONFLICTING_PATH_PROBABILITY and len(path) > 1:
+			i = random.randint(0, len(path) - 2)
+			if isinstance(path[i], str):
+				path[i] = 0
+			else:
+				path[i] = getRandomString()
+			remove_conflicts = True
+		else:
+			remove_conflicts = False
+
 		to_insert = [getRandomData(MAX_DEPTH) for i in range(elems)]
-		self._args = (getRandomInsertPath(self._obj_contents), to_insert)
+		self._args = (path, to_insert, remove_conflicts)
 
 	def _constructDeleteArgs(self):
 		self._args = ([getRandomDeletePath(self._obj_contents)],)
 
+	def _constructReadArgs(self):
+		path = getRandomDefinitePath(self._obj_contents)
+		if random.random() < READ_BY_MASK_PROBABILITY:
+			elems = random.randint(1, MAX_ELEMENTS_NUM)
+			masks = [getRandomMask(getNodeByPath(self._obj_contents, path)) for i in range(elems)]
+		else:
+			masks = None
+
+		self._args = (path, masks)
+
+	def getMethod(self):
+		return self._method
+
 	def __call__(self, conn, obj_id):
 		args = self._args
 		kwds = self._kwds
-		getattr(conn, self._method)(obj_id, *args, **kwds)
+		return getattr(conn, self._method)(obj_id, *args, **kwds)
 
 	def __str__(self):
 		return self._method + repr(self._args)
@@ -220,6 +308,7 @@ def _runTests(objects, actions, verbosity):
 
 	objs = []
 	fake_objs = []
+	times = {}
 
 	# create objects
 	for i in range(objects):
@@ -256,20 +345,29 @@ def _runTests(objects, actions, verbosity):
 
 			# create random action and test it on fake object
 			action = RandomAction(state_before)
-			action(fake_conn, fake_objs[i])
+			fake_result = action(fake_conn, fake_objs[i])
 
 			# if object gets deleted, return its state to original
 			fake_state_after = fake_conn.read(fake_objs[i])
 			if fake_state_after is None:
-				fake_conn.modify(fake_objs[i], fake_state_before, [])
+				fake_conn.modify(fake_objs[i], [], fake_state_before)
 				continue
 
 			# try to perform action on a real object
 			try:
-				action(conn, objs[i])
+				starting_time = time.time()
+				result = action(conn, objs[i])
+				action_time = time.time() - starting_time
+
+				method = action.getMethod()
+				if method not in times.keys():
+					times[method] = action_time
+				else:
+					times[method] += action_time
+
 			except:
 				print("Error performing action on object " + str(i) +
-					": " + action.dump())
+					": " + action.dump(verbosity))
 				if verbosity > 3:
 					print("State before: " + str(fake_state_before))
 					conn._engine.dump()
@@ -301,11 +399,24 @@ def _runTests(objects, actions, verbosity):
 					conn._engine.dump()
 				raise Exception("Functionality error")
 
+			# compare action return values (if any)
+			if result != fake_result:
+				print("Action return values are different:")
+				print("State before: " + repr(fake_state_before))
+				print("Action: " + action.dump(verbosity))
+				print("Main return value: " + repr(result))
+				print("Fake return value: " + repr(fake_result))
+				if verbosity > 3:
+					conn._engine.dump()
+				raise Exception("Functionality error")
 
-def runFuzzTest(objects=1, actions=100, verbosity=2, seed=0):
+	return times
+
+def runFuzzTest(objects=1, actions=100, verbosity=2, seed=0, show_report=True):
 	"""Main test function. Start test, terminate on first exception"""
 
-	print("Fuzz test")
+	if show_report:
+		print("Fuzz test")
 
 	if seed != 0:
 		random.seed(seed)
@@ -314,20 +425,26 @@ def runFuzzTest(objects=1, actions=100, verbosity=2, seed=0):
 		random.seed()
 		state = "random seed"
 
-	print(str(objects) + " objects, " + str(actions) + " actions, " + state)
+	if show_report:
+		print(str(objects) + " objects, " + str(actions) + " actions, " + state)
+		print("=" * 70)
 
-	print("=" * 70)
-	time1 = time.time()
 	try:
-		_runTests(objects, actions, verbosity)
+		times = _runTests(objects, actions, verbosity)
 	except:
-		err_class, err_obj, err_tb = err
-		self.__stream.writeln("! " + str(err_obj))
+		times = None
+		err_class, err_obj, err_tb = sys.exc_info()
+		print("Error during test: " + str(err_obj))
 
 		# report traceback
 		if verbosity > 1:
-			traceback.print_tb(err_tb, None, self.__stream)
+			traceback.print_tb(err_tb)
 	finally:
-		print("=" * 70)
-		time2 = time.time()
-		print("Finished in {0:.3f} seconds".format(time2 - time1))
+		if show_report:
+			print("=" * 70)
+
+	if show_report and times is not None:
+		time_strings = ["- " + action + ": {0:.3f} s".format(times[action]) for action in times]
+		print("Action times:\n" + "\n".join(time_strings))
+
+	return times
