@@ -4,7 +4,8 @@ import unittest
 
 import brain
 
-from public.requests import TestRequest
+import helpers
+from public.requests import TestRequest, getParameterized
 
 class Read(TestRequest):
 	"""Test operation of ReadRequest"""
@@ -134,6 +135,11 @@ class Read(TestRequest):
 		self.assertRaises(brain.LogicError, self.conn.read,
 			self.id1, ['blablabla'])
 
+	def testReadFromNonExistentListElement(self):
+		"""Check that attempt to read from non-existent list element raises LogicError"""
+		obj = self.conn.create([1, [2, 3, 4]])
+		self.assertRaises(brain.LogicError, self.conn.read, obj, [0, 0])
+
 	def testReadPathAndMasks(self):
 		"""Check that path and masks arguments ofr read() work simultaneously"""
 		obj = self.conn.create({'tracks': [{'Name': 'track 1', 'Length': 240},
@@ -142,5 +148,7 @@ class Read(TestRequest):
 		self.assertEqual(res, [{'Length': 240}, {'Length': 300}])
 
 
-def get_class():
-	return Read
+def suite(engine_params, connection_generator):
+	res = helpers.NamedTestSuite('read')
+	res.addTestCaseClass(getParameterized(Read, engine_params, connection_generator))
+	return res
